@@ -285,7 +285,6 @@ int prompt(bool verbose, CompilerOptions &cu)
             LCompilers::LocationManager lm;
             LCompilers::PassManager lpm;
             lpm.use_default_passes();
-            lpm.do_not_use_optimization_passes();
             {
                 LCompilers::LocationManager::FileLocations fl;
                 fl.in_filename = "input";
@@ -1304,9 +1303,9 @@ int compile_to_object_file_cpp(const std::string &infile,
         }
         options += " -I" + rtlib_header_dir;
         std::string cmd = CXX + " " + options + " -o " + outfile + " -c " + cppfile;
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1407,9 +1406,9 @@ int compile_to_object_file_c(const std::string &infile,
         std::string CXX = "gcc";
         std::string options = " -I" + rtlib_header_dir;
         std::string cmd = CXX + " " + options + " -o " + outfile + " -c " + cfile;
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1462,8 +1461,12 @@ int compile_to_binary_fortran(const std::string &infile,
 int link_executable(const std::vector<std::string> &infiles,
     const std::string &outfile,
     const std::string &runtime_library_dir, Backend backend,
-    bool static_executable, bool link_with_gcc, bool kokkos,
-    bool verbose, CompilerOptions &compiler_options)
+    bool static_executable, bool shared_executable,
+    bool link_with_gcc, bool kokkos, bool verbose,
+    const std::vector<std::string> &lib_dirs,
+    const std::vector<std::string> &libraries,
+    const std::vector<std::string> &linker_flags,
+    CompilerOptions &compiler_options)
 {
     /*
     The `gcc` line for dynamic linking that is constructed below:
@@ -1527,12 +1530,27 @@ int link_executable(const std::vector<std::string> &infiles,
 #endif
     size_t dot_index = outfile.find_last_of(".");
     std::string file_name = outfile.substr(0, dot_index);
-    std::string extra_runtime_linker_path;
-    if (!compiler_options.runtime_linker_paths.empty()) {
-        for (auto &s: compiler_options.runtime_linker_paths) {
-            extra_runtime_linker_path += " -Wl,-rpath," + s;
+    std::string extra_linker_flags;
+    if (!linker_flags.empty()) {
+        for (auto &s: linker_flags) {
+            extra_linker_flags += " -W" + s;
         }
     }
+    std::string extra_library_flags;
+    if (!lib_dirs.empty()) {
+        for (auto &s: lib_dirs) {
+            extra_library_flags += " -L" + s;
+        }
+    }
+    if (!libraries.empty()) {
+        for (auto &s: libraries) {
+            extra_library_flags += " -l" + s;
+        }
+    }
+    if(static_executable && shared_executable) {
+        std::cout << "Cannot use static_executable and shared_executable together" << std::endl;
+        return 10;
+    }
     if (backend == Backend::llvm) {
         std::string run_cmd = "", compile_cmd = "";
         if (t == "x86_64-pc-windows-msvc") {
@@ -1542,6 +1560,41 @@ int link_executable(const std::vector<std::string> &infiles,
             }
             compile_cmd += runtime_library_dir + "\\lfortran_runtime_static.lib";
             run_cmd = outfile;
+        } else if (LCompilers::startswith(t, "wasm")) {
+            std::string CC, options, runtime_lib;
+            if (LCompilers::endswith(t, "wasi")) {
+                char* wasi_sdk_path = std::getenv("WASI_SDK_PATH");
+                if (wasi_sdk_path == nullptr) {
+                    std::cerr << "WASI_SDK_PATH must be defined to use llvm->wasm\n";
+                    return 11;
+                }
+                CC = std::string(wasi_sdk_path) + "/bin/clang";
+                options = " --target=wasm32-wasi -nostartfiles -Wl,--entry=_start -Wl,-lwasi-emulated-process-clocks";
+                runtime_lib = "lfortran_runtime_wasm_wasi.o";
+                compile_cmd = CC + options + " -o " + outfile + " ";
+            } else if (LCompilers::endswith(t, "emscripten")) {
+                char* emsdk_path = std::getenv("EMSDK_PATH");
+                if (emsdk_path == nullptr) {
+                    std::cerr << "EMSDK_PATH must be defined to use llvm->wasm\n";
+                    return 11;
+                }
+                CC = std::string(emsdk_path) + "/upstream/emscripten/emcc";
+                options = " --target=wasm32-unknown-emscripten -sSTACK_SIZE=50mb -sINITIAL_MEMORY=256mb";
+                if (!compiler_options.emcc_embed.empty()) {
+                    options += " --embed-file " + compiler_options.emcc_embed;
+                }
+                runtime_lib = "lfortran_runtime_wasm_emcc.o";
+                compile_cmd = CC + options + " -o " + outfile +
+                     (compiler_options.wasm_html ? ".html " : " ");
+            } else {
+                std::cerr << "Unsupported target: " << t << std::endl;
+                return 10;
+            }
+            for (auto &s : infiles) {
+                compile_cmd += s + " ";
+            }
+            compile_cmd += runtime_library_dir + "/" + runtime_lib;
+            compile_cmd +=  extra_linker_flags;
         } else {
             std::string CC;
             std::string base_path = "\"" + runtime_library_dir + "\"";
@@ -1568,14 +1621,20 @@ int link_executable(const std::vector<std::string> &infiles,
                 }
                 runtime_lib = "lfortran_runtime_static";
             }
+            if (shared_executable) {
+                options += " -shared ";
+            }
             compile_cmd = CC + options + " -o " + outfile + " ";
             for (auto &s : infiles) {
                 compile_cmd += s + " ";
             }
+            if(!extra_library_flags.empty()) {
+                compile_cmd += extra_library_flags + " ";
+            }
             compile_cmd += + " -L"
                 + base_path + " -Wl,-rpath," + base_path;
-            if (!extra_runtime_linker_path.empty()) {
-                compile_cmd += extra_runtime_linker_path;
+            if (!extra_linker_flags.empty()) {
+                compile_cmd += extra_linker_flags;
             }
             compile_cmd += " -l" + runtime_lib + " -lm";
             run_cmd = "./" + outfile;
@@ -1622,15 +1681,18 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
+        if(!extra_library_flags.empty()) {
+            cmd += extra_library_flags + " ";
+        }
         cmd += " -L" + base_path
             + " -Wl,-rpath," + base_path;
-        if (!extra_runtime_linker_path.empty()) {
-            cmd += extra_runtime_linker_path;
+        if (!extra_linker_flags.empty()) {
+            cmd += extra_linker_flags;
         }
         cmd += " -l" + runtime_lib + " -lm";
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1642,6 +1704,9 @@ int link_executable(const std::vector<std::string> &infiles,
         if (static_executable) {
             options += " -static ";
         }
+        if (shared_executable) {
+            options += " -shared ";
+        }
         if (compiler_options.openmp) {
             options += " -fopenmp ";
         }
@@ -1654,10 +1719,13 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
+        if(!extra_library_flags.empty()) {
+            cmd += extra_library_flags + " ";
+        }
         cmd += " " + post_options + " -lm";
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1679,12 +1747,15 @@ int link_executable(const std::vector<std::string> &infiles,
         for (auto &s : infiles) {
             cmd += s + " ";
         }
+        if(!extra_library_flags.empty()) {
+            cmd += extra_library_flags + " ";
+        }
         cmd += " -L" + base_path
             + " -Wl,-rpath," + base_path;
         cmd += " -l" + runtime_lib + " -lm";
-	if (verbose) {
+        if (verbose) {
             std::cout << cmd << std::endl;
-	}
+        }
         int err = system(cmd.c_str());
         if (err) {
             std::cout << "The command '" + cmd + "' failed." << std::endl;
@@ -1705,6 +1776,13 @@ int link_executable(const std::vector<std::string> &infiles,
         run_cmd = "node --experimental-wasi-unstable-preview1 " + outfile + ".js";
     } else if (t == "x86_64-pc-windows-msvc") {
         run_cmd = outfile;
+    } else if (LCompilers::startswith(t, "wasm")) {
+        if (LCompilers::endswith(t, "wasi")) {
+            run_cmd = "wasmtime " + outfile + " --dir=.";
+        } else if (LCompilers::endswith(t, "emscripten")) {
+            run_cmd = "node " + outfile +
+                (compiler_options.wasm_html ? ".js" : "");
+        }
     } else {
         run_cmd = "./" + outfile;
     }
@@ -1859,47 +1937,6 @@ EMSCRIPTEN_KEEPALIVE char* emit_wasm_from_source(char *input) {
 
 #endif
 
-void _parse_linker_args(std::string& arg, std::vector<std::string>& linker_flags) {
-    if (arg == "") return;
-    std::string current_arg = "";
-    for( size_t i = 0; i < arg.size(); i++ ) {
-        char ch = arg[i];
-        if (ch != ',') {
-            current_arg.push_back(ch);
-        } else {
-            linker_flags.push_back(current_arg);
-            current_arg.clear();
-        }
-    }
-    if (!current_arg.empty()) {
-        linker_flags.push_back(current_arg);
-    }
-}
-
-void _get_rpath_from_linker_flags(std::vector<std::string>& linker_flags, CompilerOptions &co) {
-    if (!linker_flags.empty() && linker_flags[0] != "l") {
-        LCompilers::LCompilersException("Linker arguments must start with -Wl");
-        return;
-    }
-    if (linker_flags.empty()) {
-        return;
-    }
-    bool is_rpath = false;
-    for( size_t i = 1; i < linker_flags.size(); i++ ) {
-        if (linker_flags[i][0] == '-') {
-            if (linker_flags[i] == "-rpath") {
-                is_rpath = true;
-            } else {
-                is_rpath = false;
-            }
-            continue;
-        }
-        if (is_rpath) {
-            co.runtime_linker_paths.push_back(linker_flags[i]);
-        }
-    }
-}
-
 int main_app(int argc, char *argv[]) {
     int dirname_length;
     LCompilers::LFortran::get_executable_path(LCompilers::binary_executable_path, dirname_length);
@@ -1918,6 +1955,7 @@ int main_app(int argc, char *argv[]) {
     std::vector<std::string> arg_l;
     std::vector<std::string> arg_L;
     std::vector<std::string> arg_files;
+    std::string arg_standard;
     bool arg_version = false;
     bool show_prescan = false;
     bool show_tokens = false;
@@ -1937,12 +1975,16 @@ int main_app(int argc, char *argv[]) {
     bool show_fortran = false;
     bool time_report = false;
     bool static_link = false;
+    bool shared_link = false;
     std::string skip_pass;
     std::string arg_backend = "llvm";
     std::string arg_kernel_f;
     bool print_targets = false;
     bool link_with_gcc = false;
     bool fixed_form_infer = false;
+    bool cpp = false;
+    bool cpp_infer = false;
+    bool no_cpp = false;
 
     std::string arg_fmt_file;
     int arg_fmt_indent = 4;
@@ -1958,6 +2000,7 @@ int main_app(int argc, char *argv[]) {
     std::string arg_pywrap_array_order="f";
     std::vector<std::string> linker_flags;
     std::vector<std::string> f_flags;
+    std::vector<std::string> O_flags;
 
     CompilerOptions compiler_options;
     compiler_options.po.runtime_library_dir = LCompilers::LFortran::get_runtime_library_dir();
@@ -1971,11 +2014,11 @@ int main_app(int argc, char *argv[]) {
     app.add_option("files", arg_files, "Source files");
     app.add_flag("-S", arg_S, "Emit assembly, do not assemble or link");
     app.add_flag("-c", arg_c, "Compile and assemble, do not link");
-    app.add_option("-o", compiler_options.arg_o, "Specify the file to place the output into");
+    app.add_option("-o", compiler_options.arg_o, "Specify the file to place the compiler's output into");
     app.add_flag("-v", arg_v, "Be more verbose");
     app.add_flag("-E", arg_E, "Preprocess only; do not compile, assemble or link");
-    app.add_option("-l", arg_l, "Link library option");
-    app.add_option("-L", arg_L, "Library path option");
+    app.add_option("-l", arg_l, "Link library option")->allow_extra_args(false);
+    app.add_option("-L", arg_L, "Library path option")->allow_extra_args(false);
     app.add_option("-I", compiler_options.po.include_dirs, "Include path")->allow_extra_args(false);
     app.add_option("-J", compiler_options.po.mod_files_dir, "Where to save mod files");
     app.add_flag("-g", compiler_options.emit_debug_info, "Compile with debugging information");
@@ -1985,12 +2028,16 @@ int main_app(int argc, char *argv[]) {
     app.add_flag("--version", arg_version, "Display compiler version information");
     app.add_option("-W", linker_flags, "Linker flags")->allow_extra_args(false);
     app.add_option("-f", f_flags, "All `-f*` flags (only -fPIC supported for now)")->allow_extra_args(false);
+    app.add_option("-O", O_flags, "Optimization level (ignored for now)")->allow_extra_args(false);
 
     // LFortran specific options
-    app.add_flag("--cpp", compiler_options.c_preprocessor, "Enable C preprocessing");
+    app.add_flag("--cpp", cpp, "Enable C preprocessing");
+    app.add_flag("--cpp-infer", cpp_infer, "Use heuristics to infer if a file needs preprocessing");
+    app.add_flag("--no-cpp", no_cpp, "Disable C preprocessing");
     app.add_flag("--fixed-form", compiler_options.fixed_form, "Use fixed form Fortran source parsing");
     app.add_flag("--fixed-form-infer", fixed_form_infer, "Use heuristics to infer if a file is in fixed form");
-    app.add_flag("--no-prescan", arg_no_prescan, "Turn off prescan");
+    app.add_option("--std", arg_standard, "Select standard conformance (lf, f23, legacy)");
+    app.add_flag("--no-prescan", arg_no_prescan, "Turn off prescanning");
     app.add_flag("--show-prescan", show_prescan, "Show tokens for the given file and exit");
     app.add_flag("--show-tokens", show_tokens, "Show tokens for the given file and exit");
     app.add_flag("--show-ast", show_ast, "Show AST for the given file and exit");
@@ -2016,7 +2063,10 @@ int main_app(int argc, char *argv[]) {
     app.add_flag("--symtab-only", compiler_options.symtab_only, "Only create symbol tables in ASR (skip executable stmt)");
     app.add_flag("--time-report", time_report, "Show compilation time report");
     app.add_flag("--static", static_link, "Create a static executable");
+    app.add_flag("--shared", shared_link, "Create a shared executable");
+    app.add_flag("--logical-casting", compiler_options.logical_casting, "Allow logical casting");
     app.add_flag("--no-warnings", compiler_options.no_warnings, "Turn off all warnings");
+    app.add_flag("--no-style-warnings", compiler_options.disable_style, "Turn off style suggestions");
     app.add_flag("--no-error-banner", compiler_options.no_error_banner, "Turn off error banner");
     app.add_option("--error-format", compiler_options.error_format, "Control how errors are produced (human, short)")->capture_default_str();
     app.add_option("--backend", arg_backend, "Select a backend (llvm, c, cpp, x86, wasm, fortran)")->capture_default_str();
@@ -2048,6 +2098,8 @@ int main_app(int argc, char *argv[]) {
     app.add_flag("--legacy-array-sections", compiler_options.legacy_array_sections, "Enables passing array items as sections if required");
     app.add_flag("--ignore-pragma", compiler_options.ignore_pragma, "Ignores all the pragmas");
     app.add_flag("--stack-arrays", compiler_options.stack_arrays, "Allocate memory for arrays on stack");
+    app.add_flag("--wasm-html", compiler_options.wasm_html, "Generate HTML file using emscripten for LLVM->WASM");
+    app.add_option("--emcc-embed", compiler_options.emcc_embed, "Embed a given file/directory using emscripten for LLVM->WASM");
 
     /*
     * Subcommands:
@@ -2082,12 +2134,6 @@ int main_app(int argc, char *argv[]) {
     app.require_subcommand(0, 1);
     CLI11_PARSE(app, argc, argv);
     lcompilers_unique_ID = compiler_options.generate_object_code ? get_unique_ID() : "";
-    if (!linker_flags.empty()) {
-        std::string linker_flags_args = linker_flags[0];
-        linker_flags.clear();
-        _parse_linker_args(linker_flags_args, linker_flags);
-        _get_rpath_from_linker_flags(linker_flags, compiler_options);
-    }
 
     if (arg_version) {
         std::string version = LFORTRAN_VERSION;
@@ -2109,6 +2155,34 @@ int main_app(int argc, char *argv[]) {
 #endif
     }
 
+    if (arg_standard == "" || arg_standard == "lf") {
+        // The default LFortran behavior, do nothing
+    } else if (arg_standard == "f23") {
+        compiler_options.disable_style = true;
+        compiler_options.implicit_typing = true;
+        compiler_options.implicit_argument_casting = true;
+        compiler_options.implicit_interface = true;
+        compiler_options.print_leading_space = true;
+        compiler_options.logical_casting = false;
+    } else if (arg_standard == "legacy") {
+        // f23
+        compiler_options.disable_style = true;
+        compiler_options.implicit_typing = true;
+        compiler_options.implicit_argument_casting = true;
+        compiler_options.implicit_interface = true;
+        compiler_options.print_leading_space = true;
+        compiler_options.logical_casting = false;
+
+        // legacy options
+        compiler_options.fixed_form = true;
+        compiler_options.legacy_array_sections = true;
+        compiler_options.use_loop_variable_after_loop = true;
+        compiler_options.generate_object_code = true;
+    } else {
+        std::cerr << "The option `--std=" << arg_standard << "` is not supported" << std::endl;
+        return 1;
+    }
+
     compiler_options.use_colors = !arg_no_color;
     compiler_options.indent = !arg_no_indent;
     compiler_options.prescan = !arg_no_prescan;
@@ -2125,8 +2199,9 @@ int main_app(int argc, char *argv[]) {
         }
     }
 
-    if( compiler_options.po.fast ) {
-        lfortran_pass_manager.use_optimization_passes();
+    if(static_link && shared_link) {
+        std::cerr << "Options '--static' and '--shared' cannot be used together" << std::endl;
+        return 1;
     }
 
     if (fmt) {
@@ -2200,6 +2275,20 @@ int main_app(int argc, char *argv[]) {
         compiler_options.fixed_form = true;
     }
 
+    if (cpp && no_cpp) {
+        throw LCompilers::LCompilersException("Cannot use --cpp and --no-cpp at the same time");
+    } else if(cpp) {
+        compiler_options.c_preprocessor = true;
+    } else if(no_cpp) {
+        compiler_options.c_preprocessor = false;
+    // Decide if a file gets preprocessing based on the extension
+    // Gfortran does the same thing
+    } else if (cpp_infer && (endswith(arg_file, ".F90") || endswith(arg_file, ".F"))) {
+        compiler_options.c_preprocessor = true;
+    } else {
+        compiler_options.c_preprocessor = false;
+    }
+
     std::string outfile;
     std::filesystem::path basename = std::filesystem::path(arg_file).filename();
     if (compiler_options.arg_o.size() > 0) {
@@ -2356,10 +2445,12 @@ int main_app(int argc, char *argv[]) {
         }
         if (err) return err;
         return link_executable({tmp_o}, outfile, runtime_library_dir,
-                backend, static_link, link_with_gcc, true, arg_v, compiler_options);
+                backend, static_link, shared_link, link_with_gcc, true, arg_v, arg_L,
+		arg_l, linker_flags, compiler_options);
     } else {
         return link_executable(arg_files, outfile, runtime_library_dir,
-                backend, static_link, link_with_gcc, true, arg_v, compiler_options);
+                backend, static_link, shared_link, link_with_gcc, true, arg_v, arg_L,
+		arg_l, linker_flags, compiler_options);
     }
     return 0;
 }
